(*****************************************************************************
 *                                                                           *
 *      Interface graphique : adaptation de la bibliothque libgraph         *
 *                                                                           *
 *****************************************************************************)

(* Michel Quercia, le 10 aot 1997 *)

type point = {x:int; y:int};;
(* Toutes les spcifications de points sont effectues  l'aide de ce type. *)

(*
 * Les dfinitions suivantes remplacent celles du module [graphics],
 * voir [graphics.mli] ou le manuel de rfrence pour leurs descriptions.
 *)

val open_graph  : string -> unit
val close_graph : unit -> unit
val clear_graph : unit -> unit
val size_x : unit -> int
val size_y : unit -> int
;;

type color = int;;
val rgb: int -> int -> int -> color
val set_color : color -> unit
val black   : color
val white   : color
val red     : color
val green   : color
val blue    : color
val yellow  : color
val cyan    : color
val magenta : color
val background : color
val foreground : color
;;

val plot           : point -> unit
val point_color    : point -> color
val moveto         : point -> unit
val current_point  : unit  -> point
val lineto         : point -> unit
val draw_arc       : point -> int -> int -> int -> int -> unit
val draw_ellipse   : point -> int -> int -> unit
val draw_circle    : point -> int -> unit
val set_line_width : int -> unit
val draw_char      : char -> unit
val draw_string    : string -> unit
val set_font       : string -> unit
val set_text_size  : int -> unit
val text_size      : string -> int * int
val fill_rect      : point -> int -> int -> unit
val fill_poly      : point array -> unit
val fill_arc       : point -> int -> int -> int -> int -> unit
val fill_ellipse   : point -> int -> int -> unit
val fill_circle    : point -> int -> unit
;;

type status = {
  mouse      : point;         (* position de la souris                      *)
  button     : bool;          (* true si un bouton est enfonc              *)
  keypressed : bool;          (* true si une nouvelle touche est disponible *)
  key        : char }         (* le cractrre frapp                        *)

  and event =
  | Button_down               (* Appui sur un bouton                        *)
  | Button_up                 (* Relchement d'un bouton                    *)
  | Key_pressed               (* Appui sur une touche                       *)
  | Mouse_motion              (* Mouvement de la souris                     *)
  | Poll                      (* Ne pas attendre, retourner immdiatement   *)
;;

val wait_next_event : event list -> status
val mouse_pos       : unit -> point
val button_down     : unit -> bool
val read_key        : unit -> char
val key_pressed     : unit -> bool
val sound           : int -> int -> unit
;;

(*
 * Les dfinitions suivantes compltent les primitives de trac fournies par
 * [libgraph].
 *)
val curveto        : point -> point -> point -> unit
val draw_poly      : point array -> unit
val poly_of_bezier : int -> point array -> point array
val draw_bezier    : int -> point array -> unit
val fill_bezier    : int -> point array -> unit
;;
(*
 * [curveto b c d] trace une approximation de l'arc cubique de Bzier dfini
 *    par le point courant, les points de contrle [b] et [c] et le point final
 *    [d]. Le point courant est plac en [d]  la fin du trac.
 *
 * [draw_poly v] trace l'arc polygonal passant par les points de [v]. Le point
 *    courant est plac  la fin de [v].
 *
 * [poly_of_bezier d v] retourne un polygone (vecteur de points) approchant
 *    l'arc cubique par morceaux de Bzier associ aux points de [v].
 *    L'arc numro [n] a pour point de dpart [v.(3n)], pour points de contrle
 *    [v.(3n+1)] et [v.(3n+2)], et pour point final [v.(3n+3)]. La longueur de
 *    [v] doit tre un multiple de 3 plus un, sinon l'exception
 *    [Invalid_argument "poly_of_bezier"] est gnre.
 *    Le paramtre [d] contrle la qualit de l'approximation : chaque arc est
 *    divis rcursivement [d] fois de suite en deux sous-arcs par la mthode
 *    des milieux, le polygone retourn est celui constitu des points
 *    obtenus (contrle et extrmits). La primitive [curveto] utilise cet
 *    algorithme avec [d=4].
 *
 * [draw_bezier d v] et [fill_bezier d v] sont fonctionnellement quivalents 
 *    [draw_poly (poly_of_bezier d v)] et [fill_poly (poly_of_bezier d v)].
 *)

                       (* +-----------------+
                          |  Images bitmap  |
                          +-----------------+ *)


type image = Graphics.image;;
val transp        : color
val make_image    : color array array -> image
val dump_image    : image -> color array array
val draw_image    : image -> int -> int -> unit
val get_image     : int   -> int -> int -> int -> image
val create_image  : int   -> int -> image
val blit_image    : image -> int -> int -> unit
val current_image : unit -> color array array
;;

                         (* +-----------------------+
                            |  Images vectorielles  |
                            +-----------------------+ *)

type graphic_cmd =
| Set_color    	 of color
| Set_line_width of int
| Set_font       of string
| Set_text_size  of int
| Poly    	 of bool * (point array)
| Bezier 	 of bool * int * (point array)
| Arc     	 of bool * point * int * int * int * int
| Ellipse 	 of bool * point * int * int
| String  	 of point * string
;;
(*
 * commandes reconnues par la bibliothque
 *
 * le paramtre boolen pour [Poly], [Bezier], [Arc] et [Ellipse] indique
 * si le trac doit tre rempli ou non.
 *)

type vimage = graphic_cmd list;;
(* liste des commandes produisant une image *)

val set_record     : bool -> unit   (* active/dsactive l'enregistrement des commandes *)
val current_vimage : unit -> vimage (* image actuellement enregistre *)
val display_vimage : vimage -> unit (* affiche [img] sans effacer l'cran *)
;;
(*
 * [set_record true]  active l'enregistrement des commandes graphiques,
 *                    initialement l'enregistrement est activ.
 * [set_record false] le dsactive, la liste des commandes enregistres n'est
 *                    pas modifie.
 * [current_vimage()] retourne une copie des commandes enregistres.
 * [display_vimage img] excute les commandes figurant dans la liste [img].
 *)

type bounding_box = VIDE | BB of bb
and bb = {xmin:int; xmax:int; ymin:int; ymax:int};;

val bounding_box : vimage -> bounding_box;;
(* Rectangle d'encombrement d'une image *)
(* les textes sont compts avec une hauteur de 20 pixels et une largeur   *)
(* de 10 pixels par caractre. Cette estimation forfaitaire peut conduire *)
(*  une bounding-box incorrecte.                                         *)

                   (* +------------------------+
                      |  Conversion de format  |
                      +------------------------+ *)

val output_postscript : bool -> out_channel -> vimage -> unit
val output_fig        : bool -> out_channel -> vimage -> unit
val output_pstricks   : bool -> out_channel -> vimage -> unit
val output_tga        : out_channel -> color array array -> unit
val output_ppm        : out_channel -> color array array -> unit
val output_ppmraw     : out_channel -> color array array -> unit

val write_postscript  : string -> unit
val write_fig         : string -> unit
val write_pstricks    : string -> unit
val write_tga         : string -> unit
val write_ppm         : string -> unit
val write_ppmraw      : string -> unit
;;

(*
 [output_xxx [b] canal img] enregistre le code [xxx] de l'image [img] sur le
 canal de sortie [canal]. Le boolen [b] indique si les instructions
 originales doivent tre incorpores en commentaire dans  le code produit.

 [write_xxx nom] enregistre le code [xxx] de l'image courante dans le
 le fichier de nom [nom]. Pour les formats vectoriels, les instructions
 originales sont incorpores en commentaire dans  le code produit.

 formats vectoriels :
 
 xxx = postscript => postscript encapsul
 xxx = fig        => FIG-3.2
 xxx = pstricks   => macros pstricks

 formats ponctuels :

 xxx = tga        => targa 24 bits non compress
 xxx = ppm        => PPM 24 bits cod ascii
 xxx = ppmraw     => PPM 24 bits cod binaire

 Limitations :

 1. Les changements de fonte et de taille de texte sont ignors dans les
    formats vectoriels. La fonte utilise est la fonte Helvetica avec une
    taille de 10 points pour [output_postscript], la fonte par dfaut pour
    [output_fig] et [output_pstricks].

 2. Le format FIG-3.2 ne reconnait plus les cubiques de Bzier. La fonction
    [output_fig] gnre  la place des "x-splines" avec 4 fois plus de points
    de contrle de faon  limiter la dissemblance avec les courbes de
    Bzier ordinaires.

Mise  l'chelle :

[output_postscript] produit du code postscript encapsul, la taille de
l'image est spcifie dans le commentaire "%%BoundingBox". Cette taille
peut tre change par modification de la matrice de transformation courante.

[output_fig] produit une image  100 dpi. Modifier au besoin la rsolution
indique (ligne 9 du fichier produit).

[output_pstricks] produit une image  1 pixel/point. Modifier au besoin la
valeur du paramtre "unit", ligne 5 du fichier produit.
*)

